# -*- coding: utf-8 -*-

from django.conf import settings

from organizations.models import Organization, OrganizationUser, OrganizationOwner
from compat import get_user_model

User = get_user_model()


def create_organization(user, name, slug, is_active=True):
    """
    Returns a new organization, also creating an initial organization user who
    is the owner.
    """
    organization = Organization.objects.create(name=name, slug=slug, is_active=is_active)
    new_user = OrganizationUser.objects.create(organization=organization, user=user, is_admin=True)
    OrganizationOwner.objects.create(organization=organization, organization_user=new_user)

    return organization


def model_field_attr(model, model_field, attr):
    """
    Returns the specified attribute for the specified field on the model class.
    """
    fields = dict([(field.name, field) for field in model._meta.fields])
    try:
        return getattr(fields[model_field], attr)
    except Exception as ex:
        return "ERROR " + str(ex)


def set_current_organization_to_session(request, org):
    """
    Sets the current org in request.session
    """
    if request.session.get('current_organization', None) is not None:  # current_org already in session
        if request.session['current_organization'] == org.slug:  # same values, no need to update
            request.session['current_organization_modified'] = False
            request.session.modified = True
            return
    # current_org not in session or not same values
    request.session['current_organization'] = org.slug
    request.session['current_organization_modified'] = True
    request.session.modified = True
    return


def get_current_organization(request):
    """
    Retuns the current organization object if set in the session or
    user is member in only one orzanization.
    None if not multi client app.
    """
    multi_client = getattr(settings, 'AR_CRM_MULTI_CLIENT', True)
    if not multi_client:
        return None

    current_org_slug = request.session.get('current_organization')
    if request.user.is_authenticated():
        try:
            org = Organization.objects.get(users=request.user, slug=current_org_slug)
        except Organization.DoesNotExist:
            org = None
            orgs = Organization.objects.filter(users=request.user)
            if orgs.count() == 1:  # user is member of one organization
                org = orgs[0]
                set_current_organization_to_session(request, org)
    else:
        org = None

    return org


def get_users_organizations(user):
    """
    Return a list of organizations for a given user
    """
    if not user or not user.is_active or not user.is_authenticated():
        return None
    return Organization.objects.get_for_user(user).all()


def get_organization_members(organization):
    """
    returns a list of OrganizationUsers for given organization
    """
    return OrganizationUser.objects.filter(organization=organization)


def get_organization_users(organization):
    """
    Returns a list of Users for a given organization
    """
    organization_users = OrganizationUser.objects.filter(organization=organization)
    user_pk_list = []
    for organization_user in organization_users:
        user_pk_list.append(organization_user.user.pk)

    return User.objects.filter(pk__in=user_pk_list)



# -*- coding: utf-8 -*-

from django.test import TestCase
from django.test.utils import override_settings
from django.contrib.auth.models import User
from django.db import IntegrityError

from organizations.models import (Organization, OrganizationUser,
        OrganizationOwner)


@override_settings(USE_TZ=True)
class ActiveManagerTests(TestCase):

    fixtures = ['users.json', 'orgs.json']

    def test_active(self):
        self.assertEqual(3, Organization.objects.all().count())
        self.assertEqual(2, Organization.active.all().count())

    def test_by_user(self):
        user = User.objects.get(username="dave")
        self.assertEqual(3, Organization.objects.get_for_user(user).count())
        self.assertEqual(2, Organization.active.get_for_user(user).count())


@override_settings(USE_TZ=True)
class OrgModelTests(TestCase):

    fixtures = ['users.json', 'orgs.json']

    def setUp(self):
        self.kurt = User.objects.get(username="kurt")
        self.dave = User.objects.get(username="dave")
        self.krist = User.objects.get(username="krist")
        self.duder = User.objects.get(username="duder")
        self.nirvana = Organization.objects.get(name="Nirvana")
        self.foo = Organization.objects.get(name="Foo Fighters")

    def test_duplicate_members(self):
        """Ensure that a User can only have one OrganizationUser object"""
        self.assertRaises(IntegrityError, self.nirvana.add_user, self.dave)

    def test_is_member(self):
        self.assertTrue(self.nirvana.is_member(self.kurt))
        self.assertTrue(self.nirvana.is_member(self.dave))
        self.assertTrue(self.foo.is_member(self.dave))
        self.assertFalse(self.foo.is_member(self.kurt))

    def test_is_admin(self):
        self.assertTrue(self.nirvana.is_admin(self.kurt))
        self.assertTrue(self.nirvana.is_admin(self.krist))
        self.assertFalse(self.nirvana.is_admin(self.dave))
        self.assertTrue(self.foo.is_admin(self.dave))

    def test_add_user(self):
        new_guy = self.foo.add_user(self.krist)
        self.assertTrue(isinstance(new_guy, OrganizationUser))
        self.assertEqual(new_guy.organization, self.foo)

    def test_get_or_add_user(self):
        """Ensure `get_or_add_user` adds a user IFF it exists"""
        new_guy, created = self.foo.get_or_add_user(self.duder)
        self.assertTrue(isinstance(new_guy, OrganizationUser))
        self.assertEqual(new_guy.organization, self.foo)
        self.assertTrue(created)

        new_guy, created = self.foo.get_or_add_user(self.dave)
        self.assertTrue(isinstance(new_guy, OrganizationUser))
        self.assertFalse(created)

    def test_delete_owner(self):
        from organizations.exceptions import OwnershipRequired
        owner = self.nirvana.owner.organization_user
        self.assertRaises(OwnershipRequired, owner.delete)

    def test_delete_missing_owner(self):
        """Ensure an org user can be deleted when there is no owner"""
        org = Organization.objects.create(name="Some test", slug="some-test")
        # Avoid the Organization.add_user method which would make an owner
        org_user = OrganizationUser.objects.create(user=self.kurt,
                organization=org)
        # Just make sure it doesn't raise an error
        org_user.delete()

    def test_nonmember_owner(self):
        from organizations.exceptions import OrganizationMismatch
        foo_user = self.foo.owner
        self.nirvana.owner = foo_user
        self.assertRaises(OrganizationMismatch, self.nirvana.owner.save)


@override_settings(USE_TZ=True)
class OrgDeleteTests(TestCase):

    fixtures = ['users.json', 'orgs.json']

    def test_delete_account(self):
        """Ensure Users are not deleted on the cascade"""
        self.assertEqual(3, OrganizationOwner.objects.all().count())
        self.assertEqual(4, User.objects.all().count())
        scream = Organization.objects.get(name="Scream")
        scream.delete()
        self.assertEqual(2, OrganizationOwner.objects.all().count())
        self.assertEqual(4, User.objects.all().count())

    def test_delete_orguser(self):
        """Ensure the user is not deleted on the cascade"""
        krist = User.objects.get(username="krist")
        org_user = OrganizationUser.objects.filter(
                organization__name="Nirvana", user=krist)
        org_user.delete()
        self.assertTrue(krist.pk)



# -*- coding: utf-8 -*-

from django.conf import settings
from django.contrib.auth import SESSION_KEY, BACKEND_SESSION_KEY
from django.contrib.auth.models import AnonymousUser
from django.utils.importlib import import_module


def request_factory_login(factory, user=None):
    """Based on this gist: https://gist.github.com/964472"""
    engine = import_module(settings.SESSION_ENGINE)
    request = factory.request()
    # request.session = engine.SessionStore()
    # request.session[SESSION_KEY] = user.id
    request.user = user or AnonymousUser()
    return request



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from os.path import join, dirname
from setuptools import setup, find_packages
import organizations as app


def long_description():
    try:
        return open(join(dirname(__file__), 'README.rst')).read()
    except IOError:
        return "LONG_DESCRIPTION Error"

setup(
    author="Ben Lopatin + arteria GmbH",
    author_email="ben.lopatin@wellfireinteractive.com",
    name='django-ar-organizations',
    version=app.__version__,
    description='Group accounts for Django',
    long_description=long_description(),
    url='https://github.com/wellfire/django-organizations/',
    license='BSD License',
    platforms=['OS Independent'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    install_requires=[
        'Django>=1.4',
        'django-extensions>=0.9',
        'django-compat>=1.0.2',
    ],
    # test_suite='tests.runtests.runtests',
    include_package_data=True,
    packages=find_packages(exclude=["tests.tests", "tests.test_app", "tests"]),
    zip_safe=False
)
